@@ -1,7 +1,8 @@
 use crate::checkers::ast::Checker;
 use crate::preview::is_fix_os_rename_enabled;
 use crate::rules::flake8_use_pathlib::helpers::{
-    check_os_pathlib_two_arg_calls, is_keyword_only_argument_non_default,
+    check_os_pathlib_two_arg_calls, has_unknown_keywords_or_starred_expr,
+    is_keyword_only_argument_non_default,
 };
 use crate::{FixAvailability, Violation};
 use ruff_macros::{ViolationMetadata, derive_message_formats};
@@ -46,6 +47,7 @@ use ruff_python_ast::ExprCall;
 /// - [Why you should be using pathlib](https://treyhunner.com/2018/12/why-you-should-be-using-pathlib/)
 /// - [No really, pathlib is great](https://treyhunner.com/2019/01/no-really-pathlib-is-great/)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.231")]
 pub(crate) struct OsRename;
 
 impl Violation for OsRename {
@@ -79,13 +81,11 @@ pub(crate) fn os_rename(checker: &Checker, call: &ExprCall, segments: &[&str]) {
         return;
     }
 
-    check_os_pathlib_two_arg_calls(
-        checker,
-        call,
-        "rename",
-        "src",
-        "dst",
-        is_fix_os_rename_enabled(checker.settings()),
-        OsRename,
-    );
+    let fix_enabled = is_fix_os_rename_enabled(checker.settings())
+        && !has_unknown_keywords_or_starred_expr(
+            &call.arguments,
+            &["src", "dst", "src_dir_fd", "dst_dir_fd"],
+        );
+
+    check_os_pathlib_two_arg_calls(checker, call, "rename", "src", "dst", fix_enabled, OsRename);
 }
