@@ -40,10 +40,7 @@ impl super::SyncNotificationHandler for DidOpenNotebook {
         session.open_notebook_document(uri.clone(), notebook);
 
         // publish diagnostics
-        let snapshot = session
-            .take_snapshot(uri)
-            .expect("snapshot should be available");
-        publish_diagnostics_for_document(&snapshot, client)?;
+        publish_diagnostics_for_document(session, &uri, client)?;
 
         Ok(())
     }
