@@ -1,5 +1,6 @@
 use crate::Db;
 use bitflags::bitflags;
+use itertools::Itertools;
 use ruff_db::files::File;
 use ruff_db::parsed::parsed_module;
 use ruff_python_ast as ast;
@@ -13,9 +14,8 @@ use ruff_python_ast::{
 use ruff_text_size::{Ranged, TextLen, TextRange};
 use std::ops::Deref;
 use ty_python_semantic::{
-    HasType, SemanticModel,
-    semantic_index::definition::DefinitionKind,
-    types::{Type, definition_kind_for_name},
+    HasType, SemanticModel, semantic_index::definition::DefinitionKind, types::Type,
+    types::ide_support::definition_kind_for_name,
 };
 
 // This module walks the AST and collects a set of "semantic tokens" for a file
@@ -337,9 +337,7 @@ impl<'db> SemanticTokenVisitor<'db> {
 
         match ty {
             Type::ClassLiteral(_) => (SemanticTokenType::Class, modifiers),
-            Type::NonInferableTypeVar(_) | Type::TypeVar(_) => {
-                (SemanticTokenType::TypeParameter, modifiers)
-            }
+            Type::TypeVar(_) => (SemanticTokenType::TypeParameter, modifiers),
             Type::FunctionLiteral(_) => {
                 // Check if this is a method based on current scope
                 if self.in_class_scope {
@@ -470,13 +468,6 @@ impl<'db> SemanticTokenVisitor<'db> {
         self.classify_from_type_and_name_str(ty, local_name.id.as_str())
     }
 
-    fn visit_type_annotation(&mut self, annotation: &ast::Expr) {
-        let prev_in_type_annotation = self.in_type_annotation;
-        self.in_type_annotation = true;
-        self.visit_expr(annotation);
-        self.in_type_annotation = prev_in_type_annotation;
-    }
-
     // Visit parameters for a function or lambda expression and classify
     // them as parameters, selfParameter, or clsParameter as appropriate.
     fn visit_parameters(
@@ -486,7 +477,20 @@ impl<'db> SemanticTokenVisitor<'db> {
     ) {
         let mut param_index = 0;
 
-        for any_param in parameters {
+        // The `parameters.iter` method does return the parameters in sorted order but only if
+        // the AST is well-formed, but e.g. not for:
+        // ```py
+        // def foo(self, **key, value):
+        //     return
+        // ```
+        // Ideally, the ast would use a single vec for all parameters to avoid this issue as
+        // discussed here https://github.com/astral-sh/ruff/issues/14315 and
+        // here https://github.com/astral-sh/ruff/blob/71f8389f61a243a0c7584adffc49134ccf792aba/crates/ruff_python_parser/src/parser/statement.rs#L3176-L3179
+        let parameters_by_start = parameters
+            .iter()
+            .sorted_by_key(ruff_text_size::Ranged::start);
+
+        for any_param in parameters_by_start {
             let parameter = any_param.as_parameter();
 
             let token_type = match any_param {
@@ -518,7 +522,11 @@ impl<'db> SemanticTokenVisitor<'db> {
 
             // Handle parameter type annotations
             if let Some(annotation) = &parameter.annotation {
-                self.visit_type_annotation(annotation);
+                self.visit_annotation(annotation);
+            }
+
+            if let Some(default) = any_param.default() {
+                self.visit_expr(default);
             }
         }
     }
@@ -570,7 +578,7 @@ impl SourceOrderVisitor<'_> for SemanticTokenVisitor<'_> {
 
                 // Handle return type annotation
                 if let Some(returns) = &func.returns {
-                    self.visit_type_annotation(returns);
+                    self.visit_annotation(returns);
                 }
 
                 // Clear the in_class_scope flag so inner functions
@@ -617,21 +625,6 @@ impl SourceOrderVisitor<'_> for SemanticTokenVisitor<'_> {
                 self.visit_body(&class.body);
                 self.in_class_scope = prev_in_class;
             }
-            ast::Stmt::AnnAssign(assign) => {
-                // Handle annotated assignments (e.g., x: int = 5)
-                if let ast::Expr::Name(name) = assign.target.as_ref() {
-                    let (token_type, modifiers) = self.classify_name(name);
-                    self.add_token(name, token_type, modifiers);
-                }
-
-                // Handle the type annotation
-                self.visit_type_annotation(&assign.annotation);
-
-                // Handle the value if present
-                if let Some(value) = &assign.value {
-                    self.visit_expr(value);
-                }
-            }
             ast::Stmt::Import(import) => {
                 for alias in &import.names {
                     if let Some(asname) = &alias.asname {
@@ -693,6 +686,13 @@ impl SourceOrderVisitor<'_> for SemanticTokenVisitor<'_> {
         }
     }
 
+    fn visit_annotation(&mut self, expr: &'_ Expr) {
+        let prev_in_type_annotation = self.in_type_annotation;
+        self.in_type_annotation = true;
+        self.visit_expr(expr);
+        self.in_type_annotation = prev_in_type_annotation;
+    }
+
     fn visit_expr(&mut self, expr: &Expr) {
         match expr {
             ast::Expr::Name(name) => {
@@ -832,23 +832,25 @@ impl SourceOrderVisitor<'_> for SemanticTokenVisitor<'_> {
         );
 
         // Visit bound expression (for TypeVar)
+        // TODO: We don't call `walk_type_param` here, because, as of today (20th Oct 2025),
+        // `walk_type_param` calls `visit_expr` instead of `visit_annotation`.
         match type_param {
             TypeParam::TypeVar(type_var) => {
                 if let Some(bound) = &type_var.bound {
-                    self.visit_type_annotation(bound);
+                    self.visit_annotation(bound);
                 }
                 if let Some(default) = &type_var.default {
-                    self.visit_type_annotation(default);
+                    self.visit_annotation(default);
                 }
             }
             TypeParam::ParamSpec(param_spec) => {
                 if let Some(default) = &param_spec.default {
-                    self.visit_type_annotation(default);
+                    self.visit_annotation(default);
                 }
             }
             TypeParam::TypeVarTuple(type_var_tuple) => {
                 if let Some(default) = &type_var_tuple.default {
-                    self.visit_type_annotation(default);
+                    self.visit_annotation(default);
                 }
             }
         }
@@ -1411,7 +1413,7 @@ u = List.__name__        # __name__ should be variable<CURSOR>
         "property" @ 168..176: Decorator
         "prop" @ 185..189: Method [definition]
         "self" @ 190..194: SelfParameter
-        "self" @ 212..216: Variable
+        "self" @ 212..216: TypeParameter
         "CONSTANT" @ 217..225: Variable [readonly]
         "obj" @ 227..230: Variable
         "MyClass" @ 233..240: Class
@@ -1796,19 +1798,23 @@ class BoundedContainer[T: int, U = str]:
         "T" @ 554..555: TypeParameter
         "value2" @ 557..563: Parameter
         "U" @ 565..566: TypeParameter
+        "self" @ 577..581: TypeParameter
+        "value1" @ 582..588: Variable
         "T" @ 590..591: TypeParameter
         "value1" @ 594..600: Parameter
+        "self" @ 609..613: TypeParameter
+        "value2" @ 614..620: Variable
         "U" @ 622..623: TypeParameter
         "value2" @ 626..632: Parameter
         "get_first" @ 642..651: Method [definition]
         "self" @ 652..656: SelfParameter
         "T" @ 661..662: TypeParameter
-        "self" @ 679..683: Variable
+        "self" @ 679..683: TypeParameter
         "value1" @ 684..690: Variable
         "get_second" @ 700..710: Method [definition]
         "self" @ 711..715: SelfParameter
         "U" @ 720..721: TypeParameter
-        "self" @ 738..742: Variable
+        "self" @ 738..742: TypeParameter
         "value2" @ 743..749: Variable
         "BoundedContainer" @ 798..814: Class [definition]
         "T" @ 815..816: TypeParameter [definition]
@@ -2225,4 +2231,67 @@ finally:
         "e" @ 146..147: Variable
         "#);
     }
+
+    #[test]
+    fn test_self_attribute_expression() {
+        let test = cursor_test(
+            r#"
+from typing import Self
+
+
+class C:
+    def __init__(self: Self):
+        self.annotated: int = 1
+        self.non_annotated = 1
+        self.x.test()
+        self.x()<CURSOR>
+
+
+"#,
+        );
+
+        let tokens = semantic_tokens_full_file(&test.db, test.cursor.file);
+
+        assert_snapshot!(semantic_tokens_to_snapshot(&test.db, test.cursor.file, &tokens), @r#"
+        "typing" @ 6..12: Namespace
+        "Self" @ 20..24: Variable
+        "C" @ 33..34: Class [definition]
+        "__init__" @ 44..52: Method [definition]
+        "self" @ 53..57: SelfParameter
+        "Self" @ 59..63: TypeParameter
+        "self" @ 74..78: Parameter
+        "annotated" @ 79..88: Variable
+        "int" @ 90..93: Class
+        "1" @ 96..97: Number
+        "self" @ 106..110: Parameter
+        "non_annotated" @ 111..124: Variable
+        "1" @ 127..128: Number
+        "self" @ 137..141: Parameter
+        "x" @ 142..143: Variable
+        "test" @ 144..148: Variable
+        "self" @ 159..163: Parameter
+        "x" @ 164..165: Variable
+        "#);
+    }
+
+    /// Regression test for <https://github.com/astral-sh/ty/issues/1406>
+    #[test]
+    fn test_invalid_kwargs() {
+        let test = cursor_test(
+            r#"
+def foo(self, **<CURSOR>key, value=10):
+    return
+"#,
+        );
+
+        let tokens = semantic_tokens_full_file(&test.db, test.cursor.file);
+
+        assert_snapshot!(semantic_tokens_to_snapshot(&test.db, test.cursor.file, &tokens), @r#"
+        "foo" @ 5..8: Function [definition]
+        "self" @ 9..13: Parameter
+        "key" @ 17..20: Parameter
+        "value" @ 22..27: Parameter
+        "10" @ 28..30: Number
+        "#);
+    }
 }
