@@ -36,6 +36,7 @@ bitflags::bitflags! {
         const DIAGNOSTIC_DYNAMIC_REGISTRATION = 1 << 14;
         const WORKSPACE_CONFIGURATION = 1 << 15;
         const RENAME_DYNAMIC_REGISTRATION = 1 << 16;
+        const COMPLETION_ITEM_LABEL_DETAILS_SUPPORT = 1 << 17;
     }
 }
 
@@ -158,6 +159,11 @@ impl ResolvedClientCapabilities {
         self.contains(Self::RENAME_DYNAMIC_REGISTRATION)
     }
 
+    /// Returns `true` if the client supports "label details" in completion items.
+    pub(crate) const fn supports_completion_item_label_details(self) -> bool {
+        self.contains(Self::COMPLETION_ITEM_LABEL_DETAILS_SUPPORT)
+    }
+
     pub(super) fn new(client_capabilities: &ClientCapabilities) -> Self {
         let mut flags = Self::empty();
 
@@ -314,6 +320,15 @@ impl ResolvedClientCapabilities {
             flags |= Self::WORK_DONE_PROGRESS;
         }
 
+        if text_document
+            .and_then(|text_document| text_document.completion.as_ref())
+            .and_then(|completion| completion.completion_item.as_ref())
+            .and_then(|completion_item| completion_item.label_details_support)
+            .unwrap_or_default()
+        {
+            flags |= Self::COMPLETION_ITEM_LABEL_DETAILS_SUPPORT;
+        }
+
         flags
     }
 }
